@@ -22,6 +22,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "gtksourcefileloader.h"
 #include "gtksourcebuffer.h"
 #include "gtksourcefile.h"
@@ -348,7 +352,7 @@ gtk_source_file_loader_constructed (GObject *object)
 		}
 	}
 
-	G_OBJECT_CLASS (gtk_source_file_loader_parent_class)->dispose (object);
+	G_OBJECT_CLASS (gtk_source_file_loader_parent_class)->constructed (object);
 }
 
 static void
