@@ -22,6 +22,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "gtksourcefilesaver.h"
 #include "gtksourcefile.h"
 #include "gtksourcebufferinputstream.h"
@@ -973,6 +977,7 @@ mount_cb (GFile              *file,
 	if (error != NULL)
 	{
 		g_task_return_error (saver->priv->task, error);
+		return;
 	}
 
 	check_externally_modified (saver);
@@ -1374,6 +1379,9 @@ gtk_source_file_saver_save_async (GtkSourceFileSaver     *saver,
  * properties will be updated: the location, the encoding, the newline type and
  * the compression type.
  *
+ * Since the 3.20 version, gtk_text_buffer_set_modified() is called with %FALSE
+ * if the file has been saved successfully.
+ *
  * Returns: whether the file was saved successfully.
  * Since: 3.14
  */
@@ -1416,6 +1424,12 @@ gtk_source_file_saver_save_finish (GtkSourceFileSaver  *saver,
 		}
 	}
 
+	if (ok && saver->priv->source_buffer != NULL)
+	{
+		gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (saver->priv->source_buffer),
+					      FALSE);
+	}
+
 	reset (saver);
 
 	return ok;
