@@ -19,6 +19,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "gtksourcegutter.h"
 #include "gtksourcegutter-private.h"
 #include "gtksourceview.h"
@@ -89,10 +93,6 @@ struct _GtkSourceGutterPrivate
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceGutter, gtk_source_gutter, G_TYPE_OBJECT)
 
-static gboolean on_view_draw (GtkSourceView   *view,
-                              cairo_t         *cr,
-                              GtkSourceGutter *gutter);
-
 static gboolean on_view_motion_notify_event (GtkSourceView   *view,
                                              GdkEventMotion  *event,
                                              GtkSourceGutter *gutter);
@@ -289,12 +289,6 @@ set_view (GtkSourceGutter *gutter,
 {
 	gutter->priv->view = view;
 
-	g_signal_connect_object (view,
-				 "draw",
-				 G_CALLBACK (on_view_draw),
-				 gutter,
-				 G_CONNECT_AFTER);
-
 	g_signal_connect_object (view,
 				 "motion-notify-event",
 				 G_CALLBACK (on_view_motion_notify_event),
@@ -970,9 +964,8 @@ apply_style (GtkSourceGutter *gutter,
 	 * are not set by gtk.
 	 */
 	gtk_style_context_add_class (style_context, class);
-
 	gtk_style_context_get_color (style_context,
-	                             gtk_widget_get_state_flags (GTK_WIDGET (view)),
+	                             gtk_style_context_get_state (style_context),
 	                             &fg_color);
 
 	gdk_cairo_set_source_rgba (cr, &fg_color);
@@ -1099,7 +1092,18 @@ draw_cells (GtkSourceGutter *gutter,
 
 		if (!gtk_text_iter_ends_line (&end))
 		{
-			gtk_text_iter_forward_to_line_end (&end);
+			/*
+			 * It turns out that gtk_text_iter_forward_to_line_end
+			 * is slower than jumping to the next line in the
+			 * btree index and then moving backwards a character.
+			 * We don't really care that we might be after the
+			 * newline breaking characters, since those are part
+			 * of the same line (rather than the next line).
+			 */
+			if (gtk_text_iter_forward_line (&end))
+			{
+				gtk_text_iter_backward_char (&end);
+			}
 		}
 
 		/* Possible improvement: if buffer and window coords have the
@@ -1217,10 +1221,10 @@ end_draw (GtkSourceGutter *gutter)
 	}
 }
 
-static gboolean
-on_view_draw (GtkSourceView   *view,
-              cairo_t         *cr,
-              GtkSourceGutter *gutter)
+void
+gtk_source_gutter_draw (GtkSourceGutter *gutter,
+                        GtkSourceView   *view,
+                        cairo_t         *cr)
 {
 	GdkRectangle clip;
 	GtkTextView *text_view;
@@ -1234,7 +1238,7 @@ on_view_draw (GtkSourceView   *view,
 
 	if (!get_clip_rectangle (gutter, view, cr, &clip))
 	{
-		return GDK_EVENT_PROPAGATE;
+		return;
 	}
 
 	gutter->priv->is_drawing = TRUE;
@@ -1291,8 +1295,6 @@ on_view_draw (GtkSourceView   *view,
 
 	g_array_free (renderer_widths, TRUE);
 	lines_info_free (info);
-
-	return GDK_EVENT_PROPAGATE;
 }
 
 static Renderer *
@@ -1445,17 +1447,17 @@ renderer_query_activatable (GtkSourceGutter *gutter,
 
 static gboolean
 redraw_for_window (GtkSourceGutter *gutter,
-                   GdkEventAny     *event,
-                   gboolean         act_on_window,
-                   gint             x,
-                   gint             y)
+		   GdkEvent        *event,
+		   gboolean         act_on_window,
+		   gint             x,
+		   gint             y)
 {
 	Renderer *at_x = NULL;
+	gint start = 0;
 	GList *item;
 	gboolean redraw;
-	gint start;
 
-	if (event->window != get_window (gutter) && act_on_window)
+	if (event->any.window != get_window (gutter) && act_on_window)
 	{
 		return FALSE;
 	}
@@ -1484,7 +1486,7 @@ redraw_for_window (GtkSourceGutter *gutter,
 			}
 			else if (renderer_query_activatable (gutter,
 			                                     renderer,
-			                                     (GdkEvent *)event,
+			                                     event,
 			                                     x,
 			                                     y,
 			                                     NULL,
@@ -1516,7 +1518,7 @@ on_view_motion_notify_event (GtkSourceView    *view,
                              GtkSourceGutter  *gutter)
 {
 	return redraw_for_window (gutter,
-	                          (GdkEventAny *)event,
+	                          (GdkEvent *)event,
 	                          TRUE,
 	                          (gint)event->x,
 	                          (gint)event->y);
@@ -1528,7 +1530,7 @@ on_view_enter_notify_event (GtkSourceView     *view,
                             GtkSourceGutter   *gutter)
 {
 	return redraw_for_window (gutter,
-	                          (GdkEventAny *)event,
+	                          (GdkEvent *)event,
 	                          TRUE,
 	                          (gint)event->x,
 	                          (gint)event->y);
@@ -1540,7 +1542,7 @@ on_view_leave_notify_event (GtkSourceView     *view,
                             GtkSourceGutter   *gutter)
 {
 	return redraw_for_window (gutter,
-	                          (GdkEventAny *)event,
+	                          (GdkEvent *)event,
 	                          FALSE,
 	                          (gint)event->x,
 	                          (gint)event->y);
