@@ -19,12 +19,60 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "gtksourcegutterrenderer.h"
 #include "gtksourcegutterrenderer-private.h"
-#include "gtksourceview-marshal.h"
+#include "gtksourcestylescheme.h"
 #include "gtksourceview-typebuiltins.h"
 #include "gtksourceview-i18n.h"
 
+/**
+ * SECTION:gutterrenderer
+ * @Short_description: Gutter cell renderer
+ * @Title: GtkSourceGutterRenderer
+ * @See_also: #GtkSourceGutter
+ *
+ * A #GtkSourceGutterRenderer represents a column in a #GtkSourceGutter. The
+ * column contains one cell for each visible line of the #GtkTextBuffer. Due to
+ * text wrapping, a cell can thus span multiple lines of the #GtkTextView. In
+ * this case, #GtkSourceGutterRendererAlignmentMode controls the alignment of
+ * the cell.
+ *
+ * The gutter renderer must announce its #GtkSourceGutterRenderer:size. The
+ * height is determined by the text view height. The width must be determined by
+ * the gutter renderer. The width generally takes into account the entire text
+ * buffer. For instance, to display the line numbers, if the buffer contains 100
+ * lines, the gutter renderer will always set its width such as three digits can
+ * be printed, even if only the first 20 lines are shown. Another strategy is to
+ * take into account only the visible lines.  In this case, only two digits are
+ * necessary to display the line numbers of the first 20 lines. To take another
+ * example, the gutter renderer for #GtkSourceMark<!-- -->s doesn't need to take
+ * into account the text buffer to announce its width. It only depends on the
+ * icons size displayed in the gutter column.
+ *
+ * An horizontal and vertical padding can be added with
+ * gtk_source_gutter_renderer_set_padding().  The total width of a gutter
+ * renderer is its size (#GtkSourceGutterRenderer:size) plus two times the
+ * horizontal padding (#GtkSourceGutterRenderer:xpad).
+ *
+ * When the available size to render a cell is greater than the required size to
+ * render the cell contents, the cell contents can be aligned horizontally and
+ * vertically with gtk_source_gutter_renderer_set_alignment().
+ *
+ * The cells rendering occurs in three phases:
+ * - begin: the gtk_source_gutter_renderer_begin() function is called when some
+ *   cells need to be redrawn. It provides the associated region of the
+ *   #GtkTextBuffer. The cells need to be redrawn when the #GtkTextView is
+ *   scrolled, or when the state of the cells change (see
+ *   #GtkSourceGutterRendererState).
+ * - draw: gtk_source_gutter_renderer_draw() is called for each cell that needs
+ *   to be drawn.
+ * - end: finally, gtk_source_gutter_renderer_end() is called.
+ */
+
 enum
 {
 	ACTIVATE,
@@ -434,13 +482,31 @@ renderer_draw_impl (GtkSourceGutterRenderer      *renderer,
 	if (renderer->priv->background_set)
 	{
 		cairo_save (cr);
-
 		gdk_cairo_rectangle (cr, background_area);
 		gdk_cairo_set_source_rgba (cr, &renderer->priv->background_color);
-
 		cairo_fill (cr);
 		cairo_restore (cr);
 	}
+	else if ((state & GTK_SOURCE_GUTTER_RENDERER_STATE_CURSOR) != 0 &&
+		 GTK_SOURCE_IS_VIEW (renderer->priv->view) &&
+		 gtk_source_view_get_highlight_current_line (GTK_SOURCE_VIEW (renderer->priv->view)))
+	{
+		GtkStyleContext *context;
+
+		context = gtk_widget_get_style_context (GTK_WIDGET (renderer->priv->view));
+
+		gtk_style_context_save (context);
+		gtk_style_context_add_class (context, "current-line-number");
+
+		gtk_render_background (context,
+				       cr,
+				       background_area->x,
+				       background_area->y,
+				       background_area->width,
+				       background_area->height);
+
+		gtk_style_context_restore (context);
+	}
 }
 
 static void
@@ -465,22 +531,21 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_VISIBLE,
 	                                 g_param_spec_boolean ("visible",
-	                                                       _("Visible"),
-	                                                       _("Visible"),
+	                                                       "Visible",
+	                                                       "Visible",
 	                                                       TRUE,
 	                                                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	/**
 	 * GtkSourceGutterRenderer:xpad:
 	 *
-	 * The x-padding of the renderer.
-	 *
-	 **/
+	 * The left and right padding of the renderer.
+	 */
 	g_object_class_install_property (object_class,
 	                                 PROP_XPAD,
 	                                 g_param_spec_int ("xpad",
-	                                                   _("X Padding"),
-	                                                   _("The x-padding"),
+	                                                   "X Padding",
+	                                                   "The x-padding",
 	                                                   -1,
 	                                                   G_MAXINT,
 	                                                   0,
@@ -489,14 +554,13 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 	/**
 	 * GtkSourceGutterRenderer:ypad:
 	 *
-	 * The y-padding of the renderer.
-	 *
-	 **/
+	 * The top and bottom padding of the renderer.
+	 */
 	g_object_class_install_property (object_class,
 	                                 PROP_YPAD,
 	                                 g_param_spec_int ("ypad",
-	                                                   _("Y Padding"),
-	                                                   _("The y-padding"),
+	                                                   "Y Padding",
+	                                                   "The y-padding",
 	                                                   -1,
 	                                                   G_MAXINT,
 	                                                   0,
@@ -505,14 +569,15 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 	/**
 	 * GtkSourceGutterRenderer:xalign:
 	 *
-	 * The x-alignment of the renderer.
-	 *
-	 **/
+	 * The horizontal alignment of the renderer. Set to 0 for a left
+	 * alignment. 1 for a right alignment. And 0.5 for centering the cells.
+	 * A value lower than 0 doesn't modify the alignment.
+	 */
 	g_object_class_install_property (object_class,
 	                                 PROP_XALIGN,
 	                                 g_param_spec_float ("xalign",
-	                                                     _("X Alignment"),
-	                                                     _("The x-alignment"),
+	                                                     "X Alignment",
+	                                                     "The x-alignment",
 	                                                     -1,
 	                                                     1,
 	                                                     0,
@@ -521,14 +586,15 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 	/**
 	 * GtkSourceGutterRenderer:yalign:
 	 *
-	 * The y-alignment of the renderer.
-	 *
-	 **/
+	 * The vertical alignment of the renderer. Set to 0 for a top
+	 * alignment. 1 for a bottom alignment. And 0.5 for centering the cells.
+	 * A value lower than 0 doesn't modify the alignment.
+	 */
 	g_object_class_install_property (object_class,
 	                                 PROP_YALIGN,
 	                                 g_param_spec_float ("yalign",
-	                                                     _("Y Alignment"),
-	                                                     _("The y-alignment"),
+	                                                     "Y Alignment",
+	                                                     "The y-alignment",
 	                                                     -1,
 	                                                     1,
 	                                                     0,
@@ -550,9 +616,7 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 		              G_TYPE_FROM_CLASS (object_class),
 		              G_SIGNAL_RUN_LAST,
 		              G_STRUCT_OFFSET (GtkSourceGutterRendererClass, activate),
-		              NULL,
-		              NULL,
-		              _gtksourceview_marshal_VOID__BOXED_BOXED_BOXED,
+		              NULL, NULL, NULL,
 		              G_TYPE_NONE,
 		              3,
 		              GTK_TYPE_TEXT_ITER,
@@ -573,11 +637,8 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 		              G_TYPE_FROM_CLASS (object_class),
 		              G_SIGNAL_RUN_LAST,
 		              G_STRUCT_OFFSET (GtkSourceGutterRendererClass, queue_draw),
-		              NULL,
-		              NULL,
-		              g_cclosure_marshal_VOID__VOID,
-		              G_TYPE_NONE,
-		              0);
+		              NULL, NULL, NULL,
+		              G_TYPE_NONE, 0);
 
 	/**
 	 * GtkSourceGutterRenderer::query-tooltip:
@@ -599,7 +660,7 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 		              G_STRUCT_OFFSET (GtkSourceGutterRendererClass, query_tooltip),
 		              g_signal_accumulator_true_handled,
 		              NULL,
-		              _gtksourceview_marshal_BOOLEAN__BOXED_BOXED_INT_INT_OBJECT,
+		              NULL,
 		              G_TYPE_BOOLEAN,
 		              5,
 		              GTK_TYPE_TEXT_ITER,
@@ -626,9 +687,7 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 		              G_TYPE_FROM_CLASS (object_class),
 		              G_SIGNAL_RUN_LAST,
 		              G_STRUCT_OFFSET (GtkSourceGutterRendererClass, query_data),
-		              NULL,
-		              NULL,
-		              _gtksourceview_marshal_VOID__BOXED_BOXED_FLAGS,
+		              NULL, NULL, NULL,
 		              G_TYPE_NONE,
 		              3,
 		              GTK_TYPE_TEXT_ITER,
@@ -653,7 +712,7 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 		              G_STRUCT_OFFSET (GtkSourceGutterRendererClass, query_activatable),
 		              g_signal_accumulator_true_handled,
 		              NULL,
-		              _gtksourceview_marshal_BOOLEAN__BOXED_BOXED_BOXED,
+		              NULL,
 		              G_TYPE_BOOLEAN,
 		              3,
 		              GTK_TYPE_TEXT_ITER,
@@ -669,8 +728,8 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_VIEW,
 	                                 g_param_spec_object ("view",
-	                                                      _("The View"),
-	                                                      _("The view"),
+	                                                      "The View",
+	                                                      "The view",
 	                                                      GTK_TYPE_TEXT_VIEW,
 	                                                      G_PARAM_READABLE));
 
@@ -686,8 +745,8 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_ALIGNMENT_MODE,
 	                                 g_param_spec_enum ("alignment-mode",
-	                                                    _("Alignment Mode"),
-	                                                    _("The alignment mode"),
+	                                                    "Alignment Mode",
+	                                                    "The alignment mode",
 	                                                    GTK_SOURCE_TYPE_GUTTER_RENDERER_ALIGNMENT_MODE,
 	                                                    GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_CELL,
 	                                                    G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
@@ -702,8 +761,8 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_WINDOW_TYPE,
 	                                 g_param_spec_enum ("window-type",
-	                                                    _("Window Type"),
-	                                                    _("The window type"),
+	                                                    "Window Type",
+	                                                    "The window type",
 	                                                    GTK_TYPE_TEXT_WINDOW_TYPE,
 	                                                    GTK_TEXT_WINDOW_PRIVATE,
 	                                                    G_PARAM_READABLE));
@@ -711,8 +770,8 @@ gtk_source_gutter_renderer_class_init (GtkSourceGutterRendererClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_SIZE,
 	                                 g_param_spec_int ("size",
-	                                                   _("Size"),
-	                                                   _("The size"),
+	                                                   "Size",
+	                                                   "The size",
 	                                                   0,
 	                                                   G_MAXINT,
 	                                                   0,
@@ -752,9 +811,8 @@ gtk_source_gutter_renderer_init (GtkSourceGutterRenderer *self)
  *
  * Called when drawing a region begins. The region to be drawn is indicated
  * by @start and @end. The purpose is to allow the implementation to precompute
- * some state before the ::draw method is called for each cell.
- *
- **/
+ * some state before the draw method is called for each cell.
+ */
 void
 gtk_source_gutter_renderer_begin (GtkSourceGutterRenderer *renderer,
                                   cairo_t                 *cr,
@@ -792,16 +850,20 @@ gtk_source_gutter_renderer_begin (GtkSourceGutterRenderer *renderer,
  * @end: a #GtkTextIter
  * @state: a #GtkSourceGutterRendererState
  *
- * Main renderering method. Implementations should implement this method to
- * draw onto the cairo context. The @background_area indicates total area of
- * the cell (without padding or margin) to be drawn. The @cell_area indicates
- * the area where content can be drawn (text, images, etc).
+ * Main renderering method. Implementations should implement this method to draw
+ * onto the cairo context. The @background_area indicates the total area of the
+ * cell to be drawn. The @cell_area indicates the area where content can be
+ * drawn (text, images, etc).
+ *
+ * The @background_area is the @cell_area plus the padding on each side (two
+ * times the #GtkSourceGutterRenderer:xpad horizontally and two times the
+ * #GtkSourceGutterRenderer:ypad vertically, so that the @cell_area is centered
+ * inside @background_area).
  *
  * The @state argument indicates the current state of the renderer and should
  * be taken into account to properly draw the different possible states
  * (cursor, prelit, selected) if appropriate.
- *
- **/
+ */
 void
 gtk_source_gutter_renderer_draw (GtkSourceGutterRenderer      *renderer,
                                  cairo_t                      *cr,
@@ -896,9 +958,8 @@ gtk_source_gutter_renderer_query_activatable (GtkSourceGutterRenderer *renderer,
  * @area: a #GdkRectangle of the cell area where the renderer is activated
  * @event: the event that triggered the activation
  *
- * Emits the ::activate signal of the renderer. This is called from
- * #GtkSourceGutter and should never have to be called manually.
- *
+ * Emits the #GtkSourceGutterRenderer::activate signal of the renderer. This is
+ * called from #GtkSourceGutter and should never have to be called manually.
  */
 void
 gtk_source_gutter_renderer_activate (GtkSourceGutterRenderer *renderer,
@@ -918,11 +979,10 @@ gtk_source_gutter_renderer_activate (GtkSourceGutterRenderer *renderer,
  * gtk_source_gutter_renderer_queue_draw:
  * @renderer: a #GtkSourceGutterRenderer
  *
- * Emits the ::queue-draw signal of the renderer. Call this from an
- * implementation to inform that the renderer has changed such that it needs
- * to redraw.
- *
- **/
+ * Emits the #GtkSourceGutterRenderer::queue-draw signal of the renderer. Call
+ * this from an implementation to inform that the renderer has changed such that
+ * it needs to redraw.
+ */
 void
 gtk_source_gutter_renderer_queue_draw (GtkSourceGutterRenderer *renderer)
 {
@@ -940,13 +1000,12 @@ gtk_source_gutter_renderer_queue_draw (GtkSourceGutterRenderer *renderer)
  * @y: The y position of the tooltip.
  * @tooltip: a #GtkTooltip.
  *
- * Emits the ::query-tooltip signal. This function is called from
- * #GtkSourceGutter. Implementations can override the default signal handler
- * or can connect to the signal externally.
+ * Emits the #GtkSourceGutterRenderer::query-tooltip signal. This function is
+ * called from #GtkSourceGutter. Implementations can override the default signal
+ * handler or can connect to the signal externally.
  *
  * Returns: %TRUE if the tooltip has been set, %FALSE otherwise
- *
- **/
+ */
 gboolean
 gtk_source_gutter_renderer_query_tooltip (GtkSourceGutterRenderer *renderer,
                                           GtkTextIter             *iter,
@@ -984,12 +1043,12 @@ gtk_source_gutter_renderer_query_tooltip (GtkSourceGutterRenderer *renderer,
  * @end: a #GtkTextIter.
  * @state: a #GtkSourceGutterRendererState.
  *
- * Emit the ::query-data signal. This function is called to query for data
- * just before rendering a cell. This is called from the #GtkSourceGutter.
- * Implementations can override the default signal handler or can connect
- * a signal handler externally to the ::query-data signal.
- *
- **/
+ * Emit the #GtkSourceGutterRenderer::query-data signal. This function is called
+ * to query for data just before rendering a cell. This is called from the
+ * #GtkSourceGutter.  Implementations can override the default signal handler or
+ * can connect a signal handler externally to the
+ * #GtkSourceGutterRenderer::query-data signal.
+ */
 void
 gtk_source_gutter_renderer_query_data (GtkSourceGutterRenderer      *renderer,
                                        GtkTextIter                  *start,
@@ -1000,7 +1059,21 @@ gtk_source_gutter_renderer_query_data (GtkSourceGutterRenderer      *renderer,
 	g_return_if_fail (start != NULL);
 	g_return_if_fail (end != NULL);
 
-	g_signal_emit (renderer, signals[QUERY_DATA], 0, start, end, state);
+
+	/* Signal emission is relatively expensive and this code path is
+	 * frequent enough to optimize the common case where we only have the
+	 * override and no connected handlers.
+	 *
+	 * This is the same trick used by gtk_widget_draw().
+	 */
+	if (G_UNLIKELY (g_signal_has_handler_pending (renderer, signals[QUERY_DATA], 0, FALSE)))
+	{
+		g_signal_emit (renderer, signals[QUERY_DATA], 0, start, end, state);
+	}
+	else if (GTK_SOURCE_GUTTER_RENDERER_GET_CLASS (renderer)->query_data)
+	{
+		GTK_SOURCE_GUTTER_RENDERER_GET_CLASS (renderer)->query_data (renderer, start, end, state);
+	}
 }
 
 /**
@@ -1047,7 +1120,8 @@ gtk_source_gutter_renderer_get_visible (GtkSourceGutterRenderer *renderer)
  * -1, which means the values will not be changed (this allows changing only
  * one of the values).
  *
- **/
+ * @xpad is the left and right padding. @ypad is the top and bottom padding.
+ */
 void
 gtk_source_gutter_renderer_set_padding (GtkSourceGutterRenderer *renderer,
                                         gint                     xpad,
@@ -1096,7 +1170,10 @@ gtk_source_gutter_renderer_get_padding (GtkSourceGutterRenderer *renderer,
  * -1, which means the values will not be changed (this allows changing only
  * one of the values).
  *
- **/
+ * @xalign is the horizontal alignment. Set to 0 for a left alignment. 1 for a
+ * right alignment. And 0.5 for centering the cells. @yalign is the vertical
+ * alignment. Set to 0 for a top alignment. 1 for a bottom alignment.
+ */
 void
 gtk_source_gutter_renderer_set_alignment (GtkSourceGutterRenderer *renderer,
                                           gfloat                   xalign,
